{% load django_bootstrap5 %}
{% load i18n %}

<div class="card card-default">
    <div class="card-header">
        <div class="card-title mb-0">
            {% translate "User profile" %}
        </div>
    </div>

    <div class="card-body">
        <form id="aa-forum-form-userprofile-modify" class="form-horizontal" autocomplete="off" action="{% url 'aa_forum:profile_index' %}" method="post">
            {% csrf_token %}

            {% bootstrap_form form %}

            {% include "aa_forum/partials/form/required-field-hint.html" %}

            <div class="form-group aa-forum-form-group aa-forum-form-modify-userprofile float-end clearfix">
                <div class="col-xs-12">
                    <button class="btn btn-primary btn-sm" type="submit">
                        {% translate "Submit" %}
                    </button>
                </div>
            </div>
        </form>
    </div>
</div>
