{% load aa_forum %}

<link
    rel="stylesheet"
    href="{% aa_forum_static 'aa_forum/css/aa-forum.min.css' %}"
    integrity="sha512-MGjCtEv3kCfS3b37f/1aBN2H+SHRDyhDJJKYgHH4xEoIHT+O6ZApeWoFb+n0J9IvLoFpnbRfz2lYGSwIur48Fw=="
    crossorigin="anonymous"
>
