{% load i18n %}

<div class="aa-forum-search-box me-3">
    <form id="aa-forum-form-search-{{ suffix }}" class="navbar-form" autocomplete="off" action="{% url 'aa_forum:search_results' %}" method="get">
        <div class="input-group has-feedback has-search">
            <span class="input-group-text">
                <svg xmlns="http://www.w3.org/2000/svg" height="16" width="16" viewBox="0 0 512 512">
                    <!--!Font Awesome Free 6.5.1 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free Copyright 2023 Fonticons, Inc.-->
                    <path d="M416 208c0 45.9-14.9 88.3-40 122.7L502.6 457.4c12.5 12.5 12.5 32.8 0 45.3s-32.8 12.5-45.3 0L330.7 376c-34.4 25.2-76.8 40-122.7 40C93.1 416 0 322.9 0 208S93.1 0 208 0S416 93.1 416 208zM208 352a144 144 0 1 0 0-288 144 144 0 1 0 0 288z"></path>
                </svg>
            </span>

            <input type="text" class="form-control" name="q" value="{{ search_term }}" placeholder="{% translate 'Search …' %}">
        </div>
    </form>
</div>
