{% load static %}

<script
    src="{% static 'aa_forum/libs/select2/4.1.0-rc1/js/select2.min.js' %}"
    integrity="sha512-L0utuMtaCxnNhnp15hhXD3XmTiKGtEQ+rFO5Of39rFdXVg/nEsYi398pk5OYJmLw9XvmbcpxW1b8cxR7/pYGPQ=="
    crossorigin="anonymous"
></script>
