{% extends "aa_forum/base.html" %}

{% load static %}
{% load i18n %}

{% block page_title %}
    {% translate "Personal messages (New message)" as page_title %}
    {% translate "Forum" as app_title %}

    {{ page_title|title }} » {{ app_title }}
{% endblock %}

{% block aa_forum_body %}
    <div class="row">
        {% include "aa_forum/partials/personal-messages/sidebar.html" %}
        {% include "aa_forum/partials/personal-messages/new-message/form.html" %}
    </div>
{% endblock %}

{% block extra_css %}
    {% include "aa_forum/bundles/ckeditor5-css.html" %}
    {% include "aa_forum/bundles/select2-css.html" %}
    {% include "aa_forum/bundles/aa-forum-css.html" %}
{% endblock %}

{% block extra_javascript %}
    {% include "aa_forum/bundles/ckeditor5-js.html" %}
    {% include "aa_forum/bundles/select2-js.html" %}

    <script>
        $(document).ready(() => {
            $("#id_recipient").select2({
                theme: "bootstrap-5",
                minimumInputLength: 3,
                placeholder: "{% translate 'Enter the recipients name' %}",
            });
        });
    </script>
{% endblock %}
