{% load i18n %}

<div class="form-group aa-forum-form-group clearfix">
    <div class="float-{{ float|default:"end" }}">
        <p class="text-muted">
            {% translate "Fields marked with an asterisk (*) are mandatory" %}
        </p>
    </div>
</div>
