# Generated by Django 4.2.11 on 2024-03-08 15:28

# Django
from django.db import migrations, models

# CKEditor
import django_ckeditor_5.fields


class Migration(migrations.Migration):

    dependencies = [
        ("aa_forum", "0016_fix_quotation_marks"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="show_unread_topics_dashboard_widget",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="message",
            name="message",
            field=django_ckeditor_5.fields.CKEditor5Field(),
        ),
        migrations.AlterField(
            model_name="personalmessage",
            name="message",
            field=django_ckeditor_5.fields.CKEditor5Field(),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="signature",
            field=django_ckeditor_5.fields.CKEditor5Field(blank=True),
        ),
    ]
